/*
 * Copyright 2010-13 Fraunhofer ISE
 *
 * This file is part of j62056.
 * For more information visit http://www.openmuc.org
 *
 * j62056 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * j62056 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with j62056.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

import java.io.IOException;
import java.util.List;

import org.openmuc.j62056.Connection;
import org.openmuc.j62056.DataSet;

public class ReadMeter {

	public static void main(String[] args) {
		if (args.length < 1 || args.length > 4) {
			printUsage();
			System.exit(1);
		}

		String serialPortName = "";
		boolean echoHandling = false;
		int baudRateChangeDelay = 0;

		for (int i = 0; i < args.length; i++) {
			if (args[i].equals("-e")) {
				echoHandling = true;
			}
			else if (args[i].equals("-d")) {
				i++;
				if (i == args.length) {
					printUsage();
					System.exit(1);
				}
				try {
					baudRateChangeDelay = Integer.parseInt(args[i]);
				} catch (NumberFormatException e) {
					printUsage();
					System.exit(1);
				}
			}
			else {
				serialPortName = args[i];
			}
		}

		Connection connection = new Connection(serialPortName, echoHandling, baudRateChangeDelay);

		try {
			connection.open();
		} catch (IOException e) {
			System.err.println("Failed to open serial port: " + e.getMessage());
			System.exit(1);
		}

		List<DataSet> dataSets = null;
		try {
			dataSets = connection.read();
		} catch (IOException e) {
			connection.close();
			System.err.println("IOException while trying to read: " + e.getMessage());
			System.exit(1);
		}

		if (dataSets == null) {
			System.err.print("Read attempt timed out");
			connection.close();
			System.exit(1);
		}

		System.out.println("Got the following data sets:");
		for (DataSet dataSet : dataSets) {
			System.out.println("id=" + dataSet.getId() + " value=" + dataSet.getValue() + " unit=" + dataSet.getUnit());
		}

		System.out.println("Reading meter successfully finished.");
		connection.close();

	}

	private static void printUsage() {
		System.out.println("Usage: ReadMeter [-e] [-d <baudRateChangeDelay>] <serialPort>");
		System.out.println("Examples for <serialPort> are /dev/ttyS0 (Linux) or COM1 (Windows)");
		System.out.println("-e = enable handling of echos caused by optical tranceivers");
		System.out
				.println("-d <baudRateChangeDelay> = delay of baud rate change in ms. Default is 0. USB to serial converters often require a delay of up to 250ms.");
	}

}
